<?php

namespace TC\PaygateDigiseller\Payment;

use XF;
use XF\Entity\PaymentProfile;
use XF\Entity\PurchaseRequest;
use XF\Http\Request;
use XF\Mvc\Controller;
use XF\Mvc\Entity\Repository;
use XF\Mvc\Reply\Redirect;
use XF\Payment\AbstractProvider;
use XF\Payment\CallbackState;
use XF\Purchasable\Purchase;

/**
 * Class Digiseller
 * @package TC\PaygateDigiseller\Payment
 */
class Digiseller extends AbstractProvider
{
	/**
	 * @return string
	 */
	public function getTitle()
	{
		return 'Digiseller';
	}

	/**
	 * @return string
	 */
	public function getApiEndpoint()
	{
		return "https://www.oplata.info/asp/pay_wm.asp";
	}

	/**
	 * @param array $options
	 * @param array $errors
	 *
	 * @return bool
	 */
	public function verifyConfig(array &$options, &$errors = [])
	{
		if (empty($options['login'])
			|| empty($options['password'])
			|| empty($options['api_guid'])
			|| empty($options['category_id'])
			|| empty($options['merchant_id'])
		)
		{
			$errors[] = XF::phrase('tc_digiseller_you_must_provide_all_data');
		}

		if ($errors)
		{
			return false;
		}

		return true;
	}

	/**
	 * @param Controller      $controller
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase        $purchase
	 *
	 * @return Redirect
	 */
	public function initiatePayment(Controller $controller, PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$paymentProfile = $purchase->paymentProfile;

		$purchase->cost = $this->getCostWithTax($paymentProfile, $purchase->cost);
		if ($purchase->cost <= 0)
		{
			return $controller->noPermission();
		}

		$product = $this->getDigisellerRepo()->getProduct($purchase, $purchaseRequest);
		if (!$product)
		{
			return $controller->noPermission();
		}

		$productId = $this->getDigisellerRepo()->getProductId($product);

		$state = new CallbackState();

		$state->purchaseRequest = $purchaseRequest;
		$state->paymentProfile = $paymentProfile;

		$state->paymentResult = CallbackState::PAYMENT_RECEIVED;
		$state->subscriberId = XF::visitor()->email ?? null;
		$state->_POST = $product;

		$state->logType = 'info';
		$state->logMessage = 'Payment created.';

		$paymentProfile->Provider->handler->log($state);

		$endpointUrl = $this->getApiEndpoint();
		$endpointUrl .= '?' . http_build_query(
				[
					'id_d'        => $productId,
					'request_key' => $purchaseRequest->request_key,
					'failpage'    => $purchase->cancelUrl,
					'email'       => XF::visitor()->email ?? null
				]);

		return $controller->redirect($endpointUrl, '');
	}

	/**
	 * @param Controller      $controller
	 * @param PurchaseRequest $purchaseRequest
	 * @param PaymentProfile  $paymentProfile
	 * @param Purchase        $purchase
	 *
	 * @return null|Redirect
	 */
	public function processPayment(Controller $controller, PurchaseRequest $purchaseRequest, PaymentProfile $paymentProfile, Purchase $purchase)
	{
		$uniqueCode = $controller->filter('uniquecode', 'str');

		if (!empty($uniqueCode))
		{
			$purchaseData = $this->getDigisellerRepo()->getPurchaseData($purchase, $uniqueCode);

			if (!empty($purchaseData))
			{
				$state = new CallbackState();

				$state->transactionId = $purchaseData['inv'];
				$state->purchaseRequest = $purchaseRequest;
				$state->paymentProfile = $paymentProfile;
				$state->subscriberId = $purchaseData['email'];
				$state->costAmount = str_replace(",", ".", $purchaseData['amount']);

				$state->_POST = $purchaseData;

				if ($this->validateTransaction($state) && empty($purchaseData['retval']))
				{
					$state->paymentResult = CallbackState::PAYMENT_RECEIVED;
					$this->completeTransaction($state);
				}

				$paymentProfile->Provider->handler->log($state);
				return $controller->redirect($purchase->returnUrl, '');
			}
		}

		return $controller->noPermission();
	}

	/**
	 * @param Request $request
	 *
	 * @return CallbackState
	 */
	public function setupCallback(Request $request)
	{
		$state = new CallbackState();

		return $state;
	}

	/**
	 * @param CallbackState $state
	 *
	 * @return bool
	 */
	public function validateCost(CallbackState $state)
	{
		$purchaseRequest = $state->getPurchaseRequest();

		$cost = $this->getCostWithTax($state->paymentProfile, $purchaseRequest->cost_amount);

		$costValidated = (round($state->costAmount, 2) == round($cost, 2));

		if ($costValidated)
		{
			$state->paymentResult = CallbackState::PAYMENT_RECEIVED;
			return true;
		}

		$state->logType = 'error';
		$state->logMessage = 'Invalid cost amount.';

		return false;
	}

	/**
	 * @param CallbackState $state
	 */
	public function getPaymentResult(CallbackState $state)
	{
		$state->paymentResult = CallbackState::PAYMENT_RECEIVED;
	}

	/**
	 * @param CallbackState $state
	 */
	public function completeTransaction(CallbackState $state)
	{
		$purchasableHandler = $state->getPurchasableHandler();

		if ($state->paymentResult == CallbackState::PAYMENT_RECEIVED)
		{
			$purchasableHandler->completePurchase($state);
			$purchasableHandler->sendPaymentReceipt($state);
			$state->logType = 'payment';
			$state->logMessage = 'Payment received.';

		}
		else
		{
			$state->logType = 'error';
			$state->logMessage = 'Unknown error.';
		}
	}

	/**
	 * @param CallbackState $state
	 */
	public function prepareLogData(CallbackState $state)
	{
		$state->logDetails = $state->_POST;
	}

	/**
	 * @var array
	 */
	protected $supportedCurrencies = [
		"RUB", "USD", "EUR"
	];

	/**
	 * @param array $supportedCurrencies
	 */
	public function setSupportedCurrencies(array $supportedCurrencies)
	{
		$this->supportedCurrencies = $supportedCurrencies;
	}

	/**
	 * @param PaymentProfile $paymentProfile
	 * @param                $unit
	 * @param                $amount
	 * @param int            $result
	 *
	 * @return bool|int
	 */
	public function supportsRecurring(PaymentProfile $paymentProfile, $unit, $amount, &$result = self::ERR_NO_RECURRING)
	{
		$result = self::ERR_NO_RECURRING;
		return false;
	}

	/**
	 * @param PaymentProfile $paymentProfile
	 * @param                $currencyCode
	 *
	 * @return bool
	 */
	public function verifyCurrency(PaymentProfile $paymentProfile, $currencyCode)
	{
		return in_array($currencyCode, $this->supportedCurrencies);
	}

	/**
	 * @param PaymentProfile $paymentProfile
	 * @param                $amount
	 *
	 * @return float|int
	 */
	private function getCostWithTax(PaymentProfile $paymentProfile, $amount)
	{
		$paymentOptions = $paymentProfile->options;

		if (!empty($paymentOptions['tax']))
		{
			$amount += $amount * ($paymentOptions['tax'] / 100);
		}

		return $amount;
	}

	/**
	 * @return Repository | \TC\PaygateDigiseller\Repository\Digiseller
	 */
	private function getDigisellerRepo()
	{
		return XF::repository('TC\PaygateDigiseller:Digiseller');
	}
}