<?php

namespace TC\PaygateDigiseller\Repository;

use GuzzleHttp\Exception\GuzzleException;
use Psr\Http\Message\ResponseInterface;
use XF;
use XF\Entity\PaymentProfile;
use XF\Entity\PurchaseRequest;
use XF\Mvc\Entity\Repository;
use XF\Mvc\Reply;
use XF\Purchasable\Purchase;

class Digiseller extends Repository
{
	/**
	 * @return string
	 */
	protected function getApiEndpoint()
	{
		return 'https://api.digiseller.ru/api';
	}

	/**
	 * @param PaymentProfile $paymentProfile
	 *
	 * @return null | string
	 */
	public function getToken(PaymentProfile $paymentProfile)
	{
		$options = $paymentProfile->options;
		$time = time();

		$response = $this->apiCall(
			'POST',
			$this->getApiEndpoint() . '/apilogin',
			[
				'json' => [
					'login'     => $options['login'],
					'timestamp' => $time,
					'sign'      => hash("sha256", $options['password'] . $options['api_guid'] . $time)
				]
			]
		);

		$responseJson = json_decode($response->getBody()->getContents(), true);
		if (!empty($responseJson['errors']))
		{
			throw new Reply\Exception($responseJson['errors']);
		}

		return $responseJson['token'] ?: null;
	}

	/**
	 * @param Purchase        $purchase
	 *
	 * @param PurchaseRequest $purchaseRequest
	 *
	 * @return null | array
	 */
	public function getProduct(Purchase $purchase, PurchaseRequest $purchaseRequest)
	{
		$paymentProfile = $purchase->paymentProfile;

		$token = $this->getToken($paymentProfile);
		if (!$token)
		{
			throw new Reply\Exception(new Reply\Error('Empty token.'));
		}

		$productDetails = [
			'content_type' => 'digisellercode',
			'name'         => [
				[
					'locale' => XF::language()->getLanguageCode(),
					'value'  => $purchase->title
				]
			],
			'description'  => [
				[
					'locale' => XF::language()->getLanguageCode(),
					'value'  => $purchase->description ?: $purchase->title
				]
			],
			'add_info'     => [
				[
					'locale' => XF::language()->getLanguageCode(),
					'value'  => XF::phrase('purchase_request_key:') . " " . $purchaseRequest->request_key
				]
			],
			'categories'   => [
				[
					'owner'       => 0,
					'category_id' => $paymentProfile->options['category_id']
				]
			],
			'price'        => [
				'price'    => $purchase->cost,
				'currency' => $purchase->currency
			],
			'verify_code'  => [
				'auto_verify' => true,
				'verify_url'  => $this->getProcessUrl()
			]
		];

		$productCreateResponse = $this->apiCall(
			'POST',
			$this->getApiEndpoint() . '/product/create/uniquefixed',
			[
				'query' => [
					'token' => $token
				],
				'json'  => $productDetails
			]
		);

		$productCreateResponseContents = $productCreateResponse->getBody()->getContents();
		$productCreateResponseJson = json_decode($productCreateResponseContents, true);
		if (!empty($productCreateResponseJson['errors']) || empty($productCreateResponseJson['content']))
		{
			throw new Reply\Exception(new Reply\Error($productCreateResponseContents));
		}

		$productId = $productCreateResponseJson['content']['product_id'];
		$productAddCode = $this->apiCall(
			'GET',
			$this->getApiEndpoint() . '/product/content/add/code/' . $productId . '/1',
			[
				'query' => [
					'token' => $token
				]
			]
		);

		$productAddCodeContents = $productAddCode->getBody()->getContents();
		$productAddCodeJson = json_decode($productAddCodeContents, true);
		if (!empty($productAddCodeJson['errors']) || empty($productAddCodeJson['content']))
		{
			throw new Reply\Exception(new Reply\Error($productAddCodeJson['errors']));
		}

		return $productCreateResponseJson;
	}

	/**
	 * @param array $product
	 *
	 * @return mixed
	 */
	public function getProductId(array $product)
	{
		if (empty($product) || empty($product['content']['product_id']))
		{
			throw new Reply\Exception(new Reply\Error('Empty product array!'));
		}

		return $product['content']['product_id'];
	}

	/**
	 * @param Purchase $purchase
	 * @param string   $uniqueCode
	 *
	 * @return null | array
	 */
	public function getPurchaseData(Purchase $purchase, $uniqueCode)
	{
		$paymentProfile = $purchase->paymentProfile;

		$purchaseData = $this->apiCall(
			'POST',
			'https://www.oplata.info/xml/check_unique_code.asp',
			[
				'json' => [
					'id_seller'   => $paymentProfile->options['merchant_id'],
					'unique_code' => $uniqueCode,
					'sign'        => hash('md5', implode(':', [$paymentProfile->options['merchant_id'], $uniqueCode, $paymentProfile->options['password']]))
				]
			]
		);

		$purchaseDataJson = json_decode($purchaseData->getBody()->getContents(), true);
		if (!empty($purchaseDataJson['errors']))
		{
			throw new Reply\Exception($purchaseDataJson['errors']);
		}

		return $purchaseDataJson;
	}

	/**
	 * @param string $method
	 * @param string $url
	 * @param array  $options
	 *
	 * @return null|ResponseInterface
	 */
	protected function apiCall($method = 'POST', $url = '', $options = [])
	{
		$client = XF::app()->http()->client();

		$options = array_merge($options, [
			'headers'         => [
				'Accept'       => 'application/json',
				'Content-Type' => 'application/json'
			],
			'timeout'         => 10,
			'connect_timeout' => 10,
			'exceptions'      => false,
		]);

		try
		{
			$response = $client->request($method, $url, $options);
		}
		catch (GuzzleException $e)
		{
			throw new Reply\Exception(new Reply\Error($e->getMessage()));
		}

		return $response;
	}

	/**
	 * @return mixed|string
	 */
	protected function getProcessUrl()
	{
		return XF::app()->router('public')->buildLink('canonical:purchase/process');
	}
}