<?php

namespace TC\PaygateFreeKassa\XF\Payment;

use XF\Entity\PaymentProfile;
use XF\Entity\PurchaseRequest;
use XF\Mvc\Controller;
use XF\Payment\AbstractProvider;
use XF\Payment\CallbackState;
use XF\Purchasable\Purchase;

class FreeKassa extends AbstractProvider
{
	public function getTitle()
	{
		return 'Free-Kassa';
	}

	public function getApiEndpoint()
	{
		return "https://www.free-kassa.ru/merchant/cash.php";
	}

	public function verifyConfig(array &$options, &$errors = [])
	{
		if (empty($options['merchant_id']) || empty($options['secret_key']) || empty($options['secret_key_2'])) {
			$errors[] = \XF::phrase('freekassa_you_must_provide_all_data');

			return false;
		}

		if ($errors) {
			return false;
		}

		return true;
	}

	public function initiatePayment(Controller $controller, PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$paymentProfile = $purchase->paymentProfile;

		$payment = [
			'm'  => $paymentProfile->options['merchant_id'],
			'oa' => $purchase->cost,
			's'  => $paymentProfile->options['secret_key'],
			'o'  => substr(md5(time()), 0, 9),
		];

		$payment['s'] = md5(implode(':', $payment));
		$payment['us_item'] = $purchaseRequest->request_key;

		$endpointUrl = $this->getApiEndpoint();
		$endpointUrl .= '?' . http_build_query($payment);

		return $controller->redirect($endpointUrl, '');
	}

	/**
	 * @param \XF\Http\Request $request
	 *
	 * @return CallbackState
	 */
	public function setupCallback(\XF\Http\Request $request)
	{
		$state = new CallbackState();

		$state->transactionId = $request->filter('intid', 'str');
		$state->requestKey = $request->filter('us_item', 'str');
		$state->subscriberId = $request->filter('P_EMAIL', 'str');
		$state->signature = $request->filter('SIGN', 'str');
		$state->costAmount = $request->filter('AMOUNT', 'unum');
		$state->orderId = $request->filter('MERCHANT_ORDER_ID', 'str');

		$state->_POST = $_POST;

		$state->ip = $request->getIp();

		$state->httpCode = 200;

		return $state;
	}

	public function validateTransaction(CallbackState $state)
	{
		if (!$state->transactionId && !$state->requestKey) {
			$state->logType = 'info';
			$state->logMessage = 'No transaction ID or signature. No action to take.';

			return false;
		}

		return parent::validateTransaction($state);
	}

	public function validatePurchasableData(CallbackState $state)
	{
		$paymentProfile = $state->getPaymentProfile();

		$options = $paymentProfile->options;
		if (empty($options['merchant_id']) || empty($options['secret_key']) || empty($options['secret_key_2'])) {
			$state->logType = 'error';
			$state->logMessage = 'Invalid merchant_id or secret_key.';

			return false;

		}

		if (!empty($state->signature)) {
			$crc = md5($paymentProfile->options['merchant_id'] . ':' . $state->costAmount . ':' . $paymentProfile->options['secret_key_2'] . ':' . $state->orderId);
			if ($crc == $state->signature) {
				return true;
			}
		}

		return false;
	}

	public function validateCost(CallbackState $state)
	{
		$purchaseRequest = $state->getPurchaseRequest();
		$cost = $purchaseRequest->cost_amount;

		$costValidated = (round($state->costAmount, 2) == round($cost, 2));

		if (!$costValidated) {
			$state->logType = 'error';
			$state->logMessage = 'YES';

			return false;
		}

		return true;
	}

	public function getPaymentResult(CallbackState $state)
	{
		$state->paymentResult = CallbackState::PAYMENT_RECEIVED;
	}

	public function completeTransaction(CallbackState $state)
	{
		$purchasableHandler = $state->getPurchasableHandler();

		switch ($state->paymentResult) {
			case CallbackState::PAYMENT_RECEIVED:
				$purchasableHandler->completePurchase($state);
				$purchasableHandler->sendPaymentReceipt($state);
				break;
			default:
				break;
		}

		$state->logType = 'info';
		$state->logMessage = 'YES';
	}

	public function prepareLogData(CallbackState $state)
	{
		$state->logDetails = $state->_POST;
	}

	protected $supportedCurrencies = [
		"RUB"
	];

	public function verifyCurrency(PaymentProfile $paymentProfile, $currencyCode)
	{
		return (in_array($currencyCode, $this->supportedCurrencies));
	}
}