<?php

namespace TC\PaygateWebMoney\Payment;

use XF;
use XF\Entity\PaymentProfile;
use XF\Entity\PurchaseRequest;
use XF\Http\Request;
use XF\Mvc\Controller;
use XF\Payment\AbstractProvider;
use XF\Payment\CallbackState;
use XF\Purchasable\Purchase;

class WebMoney extends AbstractProvider
{
	/**
	 * @return string
	 */
	public function getTitle()
	{
		return 'WebMoney';
	}

	/**
	 * @return string
	 */
	public function getApiEndpoint()
	{
		return "https://merchant.webmoney.ru/lmi/payment_utf.asp";
	}

	/**
	 * @param array $options
	 * @param array $errors
	 *
	 * @return bool
	 */
	public function verifyConfig(array &$options, &$errors = [])
	{
		if (empty($options['wallet_id']) || empty($options['secret_key']) || empty($options['secret_key_x20']))
		{
			$errors[] = XF::phrase('tc_webmoney_you_must_provide_all_data');
		}

		if (!preg_match('#^[R,B,Z,E,U,P][0-9]{12}$#', $options['wallet_id']))
		{
			$errors[] = XF::phrase('tc_webmoney_you_have_to_enter_correct_wallet');
		}

		if (!$errors)
		{
			return true;
		}

		return false;
	}

	/**
	 * @param PaymentProfile $paymentProfile
	 * @param                $amount
	 *
	 * @return float
	 */
	private function getCostWithTax(PaymentProfile $paymentProfile, $amount)
	{
		$paymentOptions = $paymentProfile->options;

		if (!empty($paymentOptions['tax']))
		{
			$tax = str_replace(',', '.', $paymentOptions['tax']);

			$amount += $amount * (floatval($tax) / 100);
		}

		return round($amount, 2);
	}

	/**
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase        $purchase
	 *
	 * @return string
	 */
	protected function getReturnUrl(PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		return $purchase->returnUrl;
	}

	/**
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase        $purchase
	 *
	 * @return array
	 */
	public function getPaymentParams(PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$paymentProfile = $purchase->paymentProfile;
		$purchaser = $purchase->purchaser;

		$purchase->cost = $this->getCostWithTax($paymentProfile, $purchase->cost);

		$params = [
			'LMI_PAYEE_PURSE'         => $paymentProfile->options['wallet_id'],
			'LMI_PAYMENT_AMOUNT'      => $purchase->cost,
			'LMI_PAYMENT_NO'          => $purchaseRequest->purchase_request_id,
			'LMI_PAYMENT_DESC_BASE64' => base64_encode($purchase->title),
			'LMI_PAYMER_EMAIL'        => $purchaser->email,
			'LMI_RESULT_URL'          => $this->getCallbackUrl(),
			'LMI_SUCCESS_URL'         => $this->getReturnUrl($purchaseRequest, $purchase),
			'LMI_FAIL_URL'            => $purchase->cancelUrl,
			'LMI_ALLOW_SDP'           => $paymentProfile->options['sdp'] ?? '',
			'REQUEST_KEY'             => $purchaseRequest->request_key
		];

		// Sign generation
		$params['LMI_PAYMENTFORM_SIGN'] = $this->signWm($params, $paymentProfile->options['secret_key_x20'], true);

		return $params;
	}

	/**
	 * @param Controller      $controller
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase        $purchase
	 *
	 * @return XF\Mvc\Reply\Error|XF\Mvc\Reply\View
	 */
	public function initiatePayment(Controller $controller, PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$paymentProfile = $purchase->paymentProfile;

		if (!$this->verifyCurrency($paymentProfile, $purchase->currency))
		{
			return $controller->error('Incorrect currency for selected Wallet Type.');
		}

		$params = $this->getPaymentParams($purchaseRequest, $purchase);

		$viewParams = [
			'endpointUrl'           => $this->getApiEndpoint(),
			'purchaseRequest'       => $purchaseRequest,
			'paymentProfile'        => $paymentProfile,
			'paymentProfileOptions' => $paymentProfile->options,
			'purchaser'             => $purchase->purchaser,
			'purchase'              => $purchase,
			'payment'               => $params,
			'notify_url'            => $this->getCallbackUrl()
		];

		return $controller->view('TC\PaygateWebMoney:WebMoney', 'payment_initiate_tcWebmoney', $viewParams);
	}

	/**
	 * @param Request $request
	 *
	 * @return CallbackState
	 */
	public function setupCallback(Request $request)
	{
		$state = new CallbackState();

		$state->transactionId = $request->filter('LMI_SYS_TRANS_NO', 'str');
		$state->requestKey = $request->filter('REQUEST_KEY', 'str');
		$state->subscriberId = $request->filter('LMI_PAYER_PURSE', 'str');

		$state->signature = $request->filter('LMI_HASH2', 'str');
		$state->costAmount = $request->filter('LMI_PAYMENT_AMOUNT', 'unum');
		$state->costCurrency = $this->getWalletCurrencyCode($request->filter('LMI_PAYEE_PURSE', 'str'));

		$state->walletNumber = $request->filter('LMI_PAYEE_PURSE', 'str');

		$state->_POST = $_POST;

		$state->ip = $request->getIp();

		$state->httpCode = 200;

		return $state;
	}

	/**
	 * @param CallbackState $state
	 *
	 * @return bool
	 */
	public function validateTransaction(CallbackState $state)
	{
		if (!$state->transactionId || !$state->requestKey || !$state->signature)
		{
			$state->logType = 'info';
			$state->logMessage = 'Requested params are empty!';

			return false;
		}

		if ($state->walletNumber != $state->getPaymentProfile()->options['wallet_id'])
		{
			$state->logType = 'error';
			$state->logMessage = 'Invalid merchant_id(LMI_PAYEE_PURSE).';

			return false;
		}

		return parent::validateTransaction($state);
	}

	/**
	 * @param CallbackState $state
	 *
	 * @return bool
	 */
	public function validatePurchasableData(CallbackState $state)
	{
		$options = $state->getPaymentProfile()->options;

		if (empty($options['wallet_id']) || empty($options['secret_key']))
		{
			$state->logType = 'error';
			$state->logMessage = 'Invalid merchant_id or secret_key.';

			return false;
		}

		if (!empty($state->signature))
		{
			if ($this->signWm($state->_POST, $options['secret_key']))
			{
				return true;
			}
		}

		$state->logType = 'error';
		$state->logMessage = 'Invalid sign!';

		return false;
	}

	/**
	 * @param CallbackState $state
	 *
	 * @return bool
	 */
	public function validateCost(CallbackState $state)
	{
		$purchaseRequest = $state->getPurchaseRequest();

		$cost = $this->getCostWithTax($state->paymentProfile, $purchaseRequest->cost_amount);

		if (round($state->costAmount, 2) == round($cost, 2)
			&& $this->verifyCurrency($purchaseRequest->PaymentProfile, $state->costCurrency)
		)
		{
			return true;
		}

		$state->logType = 'error';
		$state->logMessage = 'Invalid cost amount.';

		return false;
	}

	/**
	 * @param CallbackState $state
	 */
	public function getPaymentResult(CallbackState $state)
	{
		$state->paymentResult = CallbackState::PAYMENT_RECEIVED;
	}

	/**
	 * @param CallbackState $state
	 */
	public function prepareLogData(CallbackState $state)
	{
		$state->logDetails = array_merge($state->_POST, [
			$state->ip
		]);
	}

	/**
	 * @param PaymentProfile $paymentProfile
	 * @param                $currencyCode
	 *
	 * @return bool
	 */
	public function verifyCurrency(PaymentProfile $paymentProfile, $currencyCode)
	{
		return ($currencyCode == $this->getWalletCurrencyCode($paymentProfile->options['wallet_id'])) && (in_array($currencyCode, $this->supportedCurrencies));
	}

	/**
	 * @var array
	 */
	protected $supportedCurrencies = [
		"USD",
		"EUR",
		"RUB",
		"BYR",
		"UAH"
	];

	/**
	 * @param $wallet
	 *
	 * @return string|null
	 */
	private function getWalletCurrencyCode($wallet)
	{
		$letter = substr($wallet, 0, 1);

		switch ($letter)
		{
			case 'R':
			case 'P':
				return 'RUB';
			case 'B':
				return 'BYR';
			case 'Z':
				return 'USD';
			case 'E':
				return 'EUR';
			case 'U':
				return 'UAH';
			default:
				return null;
		}
	}

	/**
	 * @param      $params
	 * @param      $secretKeyX20
	 * @param bool $generate
	 *
	 * @return bool|string
	 */
	private function signWm($params, $secretKeyX20, $generate = false)
	{
		if ($generate)
		{
			return hash('sha256', implode(';', [
				$params['LMI_PAYEE_PURSE'],
				$params['LMI_PAYMENT_AMOUNT'],
				$params['LMI_PAYMENT_NO'],
				$secretKeyX20,
				''
			]));
		}

		$signArray = implode(';', [
			$params['LMI_PAYEE_PURSE'],
			$params['LMI_PAYMENT_AMOUNT'],
			$params['LMI_PAYMENT_NO'],
			$params['LMI_MODE'],
			$params['LMI_SYS_INVS_NO'],
			$params['LMI_SYS_TRANS_NO'],
			$params['LMI_SYS_TRANS_DATE'],
			$secretKeyX20,
			$params['LMI_PAYER_PURSE'],
			$params['LMI_PAYER_WM']
		]);

		$generatedSign = strtoupper(hash('sha256', $signArray));

		return hash_equals($params['LMI_HASH2'], $generatedSign);
	}

	/**
	 * @param PaymentProfile $paymentProfile
	 * @param                $unit
	 * @param                $amount
	 * @param int            $result
	 *
	 * @return bool
	 */
	public function supportsRecurring(PaymentProfile $paymentProfile, $unit, $amount, &$result = self::ERR_NO_RECURRING)
	{
		$result = self::ERR_NO_RECURRING;

		return false;
	}
}