<?php

namespace TC\PaygateYandexMoney\Payment;

use XF;
use XF\Entity\PaymentProfile;
use XF\Entity\PurchaseRequest;
use XF\Http\Request;
use XF\Mvc\Controller;
use XF\Mvc\Reply\View;
use XF\Payment\AbstractProvider;
use XF\Payment\CallbackState;
use XF\Purchasable\Purchase;

class YandexMoney extends AbstractProvider
{
	/**
	 * @return string
	 */
	public function getTitle()
	{
		return 'Yandex.Money';
	}

	/**
	 * @return string
	 */
	public function getApiEndpoint()
	{
		return "https://money.yandex.ru/quickpay/confirm.xml";
	}

	/**
	 * @param array $options
	 * @param array $errors
	 *
	 * @return bool
	 */
	public function verifyConfig(array &$options, &$errors = [])
	{
		if (empty($options['wallet_id']) || empty($options['secret_key']))
		{
			$errors[] = XF::phrase('yandexm_you_must_provide_all_data');
		}

		if (empty($options['method_ac']) && empty($options['method_pc']))
		{
			$errors[] = XF::phrase('yandexm_you_must_select_payment_method');
		}

		if ($errors)
		{
			return false;
		}

		return true;
	}

	/**
	 * @param PaymentProfile $paymentProfile
	 * @param                $amount
	 *
	 * @return float|int
	 */
	private function getCostWithTax(PaymentProfile $paymentProfile, $amount)
	{
		$paymentOptions = $paymentProfile->options;

		if (!empty($paymentOptions['tax']))
		{
			$tax = str_replace(',', '.', $paymentOptions['tax']);

			$amount += $amount * (floatval($tax) / 100);
		}

		return round($amount, 2);
	}

	/**
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase        $purchase
	 *
	 * @return array
	 */
	protected function getPaymentParams(PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$paymentProfile = $purchase->paymentProfile;

		$purchase->cost = $this->getCostWithTax($paymentProfile, $purchase->cost);

		$params = [
			'quickpay-form' => "small",
			'is-inner-form' => "true",
			'receiver'      => $paymentProfile->options['wallet_id'],
			'targets'       => $purchase->title,
			'sum'           => $purchase->cost,
			'successURL'    => $purchase->returnUrl,
			'label'         => base64_encode($purchaseRequest->request_key)
		];

		return $params;
	}

	/**
	 * @param Controller      $controller
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase        $purchase
	 *
	 * @return View
	 */
	public function initiatePayment(Controller $controller, PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$paymentProfile = $purchase->paymentProfile;

		$params = $this->getPaymentParams($purchaseRequest, $purchase);

		$viewParams = [
			'endpointUrl'           => $this->getApiEndpoint(),
			'purchaseRequest'       => $purchaseRequest,
			'paymentProfile'        => $paymentProfile,
			'paymentProfileOptions' => $paymentProfile->options,
			'purchaser'             => $purchase->purchaser,
			'purchase'              => $purchase,
			'payment'               => $params,
			'notify_url'            => $this->getCallbackUrl()
		];

		return $controller->view('XF:Purchase\YandexMoneyInitiate', 'payment_initiate_tcYandexM', $viewParams);
	}

	/**
	 * @param Request $request
	 *
	 * @return CallbackState
	 */
	public function setupCallback(Request $request)
	{
		$state = new CallbackState();

		$state->transactionId = $request->filter('operation_id', 'str');
		$state->requestKey = base64_decode($request->filter('label', 'str'));
		$state->subscriberId = $request->filter('sender', 'str');
		$state->signature = $request->filter('sha1_hash', 'str');
		$state->costAmount = $request->filter('withdraw_amount', 'unum');
		$state->costCurrency = $request->filter('currency', 'str');

		$state->unaccepted = $request->filter('unaccepted', 'str');
		$state->codepro = $request->filter('codepro', 'str');
		$state->test = $request->filter('test_notification', 'str');

		$state->_POST = $_POST;

		$state->ip = $request->getIp();

		return $state;
	}

	/**
	 * @param CallbackState $state
	 *
	 * @return bool
	 */
	public function validateTransaction(CallbackState $state)
	{
		if ($state->transactionId && $state->requestKey)
		{
			if ($state->unaccepted != 'true' && $state->codepro == 'false')
			{
				return parent::validateTransaction($state);
			}

			$state->logType = 'info';
			$state->logMessage = 'Unaccepted or Code Protected! No action to take.';

			return false;
		}

		if($state->test == 'true')
		{
			$state->logType = 'info';
			$state->logMessage = '[TC] Paygate: Yandex.Money: Test passed successfully.';
		}

		return false;
	}

	/**
	 * @param CallbackState $state
	 *
	 * @return bool
	 */
	public function validatePurchaseRequest(CallbackState $state)
	{
		if (!$state->getPurchaseRequest())
		{
			return false;
		}

		return true;
	}

	/**
	 * @param CallbackState $state
	 *
	 * @return bool
	 */
	public function validatePurchasableData(CallbackState $state)
	{
		$paymentProfile = $state->getPaymentProfile();

		$options = $paymentProfile->options;
		if (!empty($options['wallet_id']) && !empty($options['secret_key']))
		{
			if (!empty($state->signature))
			{
				if ($this->ySign($state->_POST, $options['secret_key']))
				{
					return true;
				}
			}
		}

		$state->logType = 'error';
		$state->logMessage = 'Invalid merchant_id or secret_key.';

		return false;
	}

	/**
	 * @param CallbackState $state
	 *
	 * @return bool
	 */
	public function validateCost(CallbackState $state)
	{
		$purchaseRequest = $state->getPurchaseRequest();

		$cost = $this->getCostWithTax($state->paymentProfile, $purchaseRequest->cost_amount);

		$costValidated = (round($state->costAmount, 2) == round($cost, 2)
			&& $state->costCurrency == $this->currencies[$purchaseRequest->cost_currency]
		);

		if ($costValidated)
		{
			return true;
		}

		$state->logType = 'error';
		$state->logMessage = 'Invalid Cost or Currency.';

		return false;
	}

	/**
	 * @param CallbackState $state
	 */
	public function getPaymentResult(CallbackState $state)
	{
		if ($state->unaccepted != "true")
		{
			$state->paymentResult = CallbackState::PAYMENT_RECEIVED;
		}
	}

	/**
	 * @param CallbackState $state
	 */
	public function prepareLogData(CallbackState $state)
	{
		$state->logDetails = $state->_POST;
	}

	/**
	 * @var array
	 */
	protected $currencies = [
		'RUB' => 643
	];

	/**
	 * @var array
	 */
	protected $supportedCurrencies = [
		"RUB"
	];

	/**
	 * @param PaymentProfile $paymentProfile
	 * @param                $unit
	 * @param                $amount
	 * @param int            $result
	 *
	 * @return bool|int
	 */
	public function supportsRecurring(PaymentProfile $paymentProfile, $unit, $amount, &$result = self::ERR_NO_RECURRING)
	{
		$result = self::ERR_NO_RECURRING;
		return false;
	}

	/**
	 * @return array
	 */
	protected function getSupportedRecurrenceRanges()
	{
		return [];
	}

	/**
	 * @param PaymentProfile $paymentProfile
	 * @param                $currencyCode
	 *
	 * @return bool
	 */
	public function verifyCurrency(PaymentProfile $paymentProfile, $currencyCode)
	{
		return in_array($currencyCode, $this->supportedCurrencies);
	}

	/**
	 * Generating sign for Yandex.Money
	 *
	 * @param array  $details = Transaction Details Array
	 * @param string $key     = Secret Key
	 *
	 * @return bool
	 */
	private function ySign($details, $key)
	{
		$receivedSign = $details['sha1_hash'];

		$generatedSign = sha1(implode('&', [
			$details['notification_type'],
			$details['operation_id'],
			$details['amount'],
			$details['currency'],
			$details['datetime'],
			$details['sender'],
			$details['codepro'],
			$key,
			$details['label']
		]));

		return hash_equals($receivedSign, $generatedSign);
	}
}